package stftr;

public class Board implements java.io.Serializable {
    /** The height of the board game */
    public static final int HEIGHT=10;
    /** The width of the board game */
    public static final int  WIDTH=10;
    /** Position on the board is empty. (returned by getType) */
    public static final int EMPTY=0;
    /** Position on the board is occupied by a player. (returned by getType) */
    public static final int PLAYER=1;
    /** Position on the board is occupied by a wall. (returned by getType) */
    public static final int WALL=2;
    /** Player is facing up. (returned by getDirection) */
    public static final int UP=0;
    /** Player is facing down. (returned by getDirection) */
    public static final int DOWN=1;
    /** Player is facing left. (returned by getDirection) */
    public static final int LEFT=2;
    /** Player is facing right. (returned by getDirection) */
    public static final int RIGHT=3;
    
    private final int ID_MASK=0xFFFFFFF0; // requires >> 4 to have numbers from 0 to 2^28-1
    private final int TYPE_MASK=0x0000000C; // requires >> 2 to have numbers from 0 to 3
    private final int DIRECTION_MASK=0x00000003; // no shift required to have numbers from 0 to 3
    
    /** Should be read-only... */
    public int[][] board; // 32 bits -> 28 for ID, 2 for type and 2 for direction
    
    /**
    * Creates an empty board.
    */
    public Board() {
        board=new int[HEIGHT][WIDTH];
        for (int height=0; height<HEIGHT; height++)
            for (int width=0; width<WIDTH; width++)
                putBoard(width, height, EMPTY, 0, 0);
    }
    
    /**
    * Returns the type of the object.
    *
    * @param width The width...
    * @param height The height...
    * @return The type, being one of EMPTY, PLAYER or WALL.
    */
    public int getType(int width, int height) {
        int i=board[height][width];
        i&=TYPE_MASK;
        return i >> 2;
    }
    
    /**
    * Returns the ID of the object.
    *
    * @param width The width...
    * @param height The height...
    * @return The ID. A number from 0 to 2^28-1.
    */
    public int getID(int width, int height) {
        int i=board[height][width];
        i&=ID_MASK;
        return i >> 4;
    }
    
    /**
    * Returns the direction of the object in that byte.
    *
    * @param width The width...
    * @param height The height...
    * @return The type, being one of UP, DOWN, LEFT or RIGHT.
    */
    public int getDirection(int width, int height) {
        int i=board[height][width];
        return i & DIRECTION_MASK;
    }
    
    /**
    * Puts the required in the board...
    */
    public void putBoard(int width, int height, int type, int direction, int id) {
        int i=id << 4;
        i|=type << 2;
        i|=direction;
        
        board[height][width]=i;
    }
    
    /**
    * Returns a string with the representation of the Board.
    * '.' for empty, '#' for a wall and 'P' for player
    */
    public String toString() {
        String ret="";
        
        for (int height=0; height<WIDTH; height++) {
            if (height!=0)
                ret+="\n";
            
            for (int width=0; width<WIDTH; width++)
                switch(getType(width, height)) {
                    case PLAYER: ret+="P"; break;
                    case WALL: ret+="#"; break;
                    case EMPTY: ret+="."; break;
                    default: ret+="?"; break;
                }
        }
        
        return ret;
    }
}