package stftr.player;

import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import stftr.Board;
import stftr.protocols.Player.*;

/**
*  This class is responsible to graphically display a Board.
*/
public class GameBoardComponent extends Canvas implements java.io.Serializable {
    static final String IMAGES_PATH="/stftr/images/"; // don't forget last slash
    static final int WIDTH=Board.WIDTH*32;
    static final int HEIGHT=Board.HEIGHT*32;

    Graphics myGraphics;

    Board board;
    public Integer playerID;
    Hashtable images;
    java.awt.image.ImageObserver imageObserver;
    KeyListener keyListener;

    /**
    * Creates a new instance of GameBoardComponent.
    */
    GameBoardComponent() {
        keyListener=null;
        myGraphics=null;
        board=new Board();
        playerID=new Integer(-1);

        imageObserver=new Button();
        images=new Hashtable();
        checkFiles();
        images.put("wall", loadImage(IMAGES_PATH+"wall.gif"));
        images.put("playerUP", loadImage(IMAGES_PATH+"playerUP.gif"));
        images.put("playerDOWN", loadImage(IMAGES_PATH+"playerDOWN.gif"));
        images.put("playerLEFT", loadImage(IMAGES_PATH+"playerLEFT.gif"));
        images.put("playerRIGHT", loadImage(IMAGES_PATH+"playerRIGHT.gif"));        
        images.put("opponentUP", loadImage(IMAGES_PATH+"opponentUP.gif"));
        images.put("opponentDOWN", loadImage(IMAGES_PATH+"opponentDOWN.gif"));
        images.put("opponentLEFT", loadImage(IMAGES_PATH+"opponentLEFT.gif"));
        images.put("opponentRIGHT", loadImage(IMAGES_PATH+"opponentRIGHT.gif"));

        setBackground(Color.white);
        
        setSize(WIDTH, HEIGHT);
        Component p = getParent();
        if (p != null) {
            p.invalidate();
            p.doLayout();
        }
    }
    
    /**
    * Checks if image files are available. If not it calls System.exit(1).
    */
    void checkFiles() {
        if (!new java.io.File(IMAGES_PATH+"wall.gif").exists() ||
            !new java.io.File(IMAGES_PATH+"playerUP.gif").exists() ||
            !new java.io.File(IMAGES_PATH+"playerDOWN.gif").exists() ||
            !new java.io.File(IMAGES_PATH+"playerLEFT.gif").exists() ||
            !new java.io.File(IMAGES_PATH+"playerRIGHT.gif").exists() ||
            !new java.io.File(IMAGES_PATH+"opponentUP.gif").exists() ||
            !new java.io.File(IMAGES_PATH+"opponentDOWN.gif").exists() ||
            !new java.io.File(IMAGES_PATH+"opponentRIGHT.gif").exists() ||
            !new java.io.File(IMAGES_PATH+"opponentRIGHT.gif").exists()
            ) {
            System.err.println("[GameBoardComponent] Could not find image file(s)");
            System.exit(1);
        }
    }

    /**
    * Loads an image. Blocks until image if fully available for display.
    *
    * @param name The filename of the image.
    * @return An Image instance with the image requested.
    */
    Image loadImage(String name) {
        System.out.print("Buffering ["+name+"]");
        try {
            Image img=Toolkit.getDefaultToolkit().createImage(name);

            synchronized(img){
                try {
                    while (!Toolkit.getDefaultToolkit().prepareImage(img, -1, -1, imageObserver)) {
                        img.wait(100);
                    }
                } catch (Exception e) {}
            }

            System.out.println(" [DONE]");
            return img;
        } catch (Exception e) {
            System.out.println(" [ERROR]");
            e.printStackTrace();
            return null;
        }
    }
    
    /**
    * Gets the preferred size of this component. 
    * (overwritting from java.awt.Component)
    *
    * @return A dimension object indicating this component's preferred size.
    */
    public Dimension getPreferredSize() {
        return new Dimension(WIDTH, HEIGHT);
    }

    /**
    * Paints the board.
    * (overwritting from java.awt.Canvas)
    */
    public synchronized void paint(Graphics g) {
        for (int y=0; y<Board.HEIGHT; y++)
            for (int x=0; x<Board.WIDTH; x++)
                if (board.getType(x,y)!=Board.EMPTY)
                    g.drawImage(getImage(x, y) , x*32, y*32, this);
    }

    /**
    * Returns an Image representing what is in the board in a given position.
    * Returns null on EMPTY (also on unknow type). 
    *
    * @param x The height on the board. 0&gt;x&gt;Board.HEIGHT-1
    * @param y The width on the board. 0&gt;y&gt;Board.WIDTH-1
    */
    Image getImage(int x, int y) {
        switch (board.getType(x, y)) {
            case Board.WALL:
                return (Image) images.get("wall");

            case Board.PLAYER:
                String key;

                if (playerID.equals(new Integer(board.getID(x, y))))
                    key="player";
                else
                    key="opponent";
                switch (board.getDirection(x, y)) {
                    case Board.UP: key+="UP"; break;
                    case Board.DOWN: key+="DOWN"; break;
                    case Board.LEFT: key+="LEFT"; break;
                    case Board.RIGHT: key+="RIGHT"; break;
                }

                return (Image) images.get(key);

        }

        return null;
    }

    /**
    * Changes the board. This method causes this component to repaint.
    *
    * @param board The new board.
    */
    public void setBoard(Board board) {
        Graphics g=getGraphics();
        Board oldBoard=this.board;
        this.board=board;

        for (int y=0; y<Board.HEIGHT; y++)
            for (int x=0; x<Board.WIDTH; x++)
                if (board.board[y][x]!=oldBoard.board[y][x]) {
                    Image img=getImage(x, y);
                    if (img==null) { // empty
                        g.setColor(Color.white);
                        g.fillRect(x*32, y*32, 32, 32);
                    } else
                        g.drawImage(img, x*32, y*32, this);
                }
    }

    /**
    * Sets this component key listener...
    */
    public void setKeyListener(KeyListener keyListener) {
        this.keyListener=keyListener;
    }

    /**
    * Starts to listen the events...
    */
    public void startListen() {
        addKeyListener(keyListener);
    }

    /**
    * Stops listenning events...
    */
    public void stopListen() {
        removeKeyListener(keyListener);
    }
}