package stftr.player;

import appia.*;
import appia.protocols.tcpcomplete.*;
import java.awt.*;
import java.awt.event.*;
import stftr.Board;
import stftr.protocols.Player.*;

/**
* This class is responsible to "listen" to the user clicking to close the
* window and thus making a clean exit of the program.
*/
class GamePlayerWindowAdapter extends WindowAdapter {

    /**
    * Shuts down the program.
    */
    public void windowClosing(WindowEvent e) {
        System.exit(0);
    }
}

/**
* This class is responsible to "listen" to the user clicking to connect to
* the game server.
*/
class ConnectButtonListener implements ActionListener {
    Button b;
    PlayerSession ps;
    String host;
    int port;
    GameBoardComponent boardComponent;

    /**
    * Instantiates a new ConnectButtonListener.
    *
    * @param b The [connect to server] button to listen
    * @param ps The PlayerSession used to connect to the server
    * @param host The game server host name
    * @param port The game server port
    * @param boardComponent The BoardComponent that is given focus after requesting the connection
    */
    ConnectButtonListener(Button b, PlayerSession ps, String host, int port, GameBoardComponent boardComponent) {
        this.b=b;
        this.ps=ps;
        this.host=host;
        this.port=port;
        this.boardComponent=boardComponent;
    }

    /**
    * Sends a request to connect to the server.
    */
    public void actionPerformed(ActionEvent e) {
        ps.connectToServer(host, port);
        b.setEnabled(false);
        boardComponent.requestFocus();
    }
}

/**
* This class is responsible to "listen" to the user clicking to [quit] button
* and thus making a clean exit of the program.
*/
class QuitButtonListener implements ActionListener {
    /**
    * Shuts down the program.
    */
    public void actionPerformed(ActionEvent e) {
        System.exit(0);
    }
}

/**
* Listens to the key pressed and if is an action key, requests to the server
* the desired action.
*/
class MyKeyListener implements KeyListener {
    PlayerSession playerSession;

    /**
    * Instantiates a MyKeyListener.
    *
    * @param playerSession The PlayerSession to were requests are made
    */
    public MyKeyListener(PlayerSession playerSession) {
        this.playerSession=playerSession;
    }

    /**
    * Checks what the key pressed and request the associated action.
    */
    public void keyPressed(KeyEvent e) {
        switch (e.getKeyCode()) {
            case KeyEvent.VK_UP:
                playerSession.doAction(PlayerSession.UP);
                break;

            case KeyEvent.VK_DOWN:
                playerSession.doAction(PlayerSession.DOWN);
                break;

            case KeyEvent.VK_LEFT:
                playerSession.doAction(PlayerSession.LEFT);
                break;

            case KeyEvent.VK_RIGHT:
                playerSession.doAction(PlayerSession.RIGHT);
                break;

            case KeyEvent.VK_SPACE:
                playerSession.doAction(PlayerSession.SHOOT);
                break;
        }
    }

    public void keyTyped(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}
}

/**
* This class is the player application...
*/
public class GamePlayer extends Frame {
    private final static int WIDTH=GameBoardComponent.WIDTH+20, HEIGHT=GameBoardComponent.HEIGHT+100;
    GameBoardComponent boardComponent;
    Button connectButton, quitButton;

    public GamePlayer(String host, int port) {
        super("STFTR Game Player");
        setSize(WIDTH, HEIGHT);
        addWindowListener(new GamePlayerWindowAdapter()); 

        setLayout(new FlowLayout());

        boardComponent=new GameBoardComponent();
        add(boardComponent);

        Layer[] qosLayers={
            new TcpCompleteLayer(),
            new PlayerLayer(boardComponent)
        };

        QoS qos=null;
        
        try {
            qos=new QoS("stftr game player QoS", qosLayers);
        } catch(AppiaInvalidQoSException ex) {
            System.err.println("Invalid QoS");
            System.err.println(ex.getMessage());
            System.exit(1);
        }

        Channel c=qos.createUnboundChannel("stftr channel");
        Session s=qosLayers[qosLayers.length-1].createSession();

        ChannelCursor cc=c.getCursor();
        
        try {
            cc.top();
            cc.setSession(s);
        } catch(AppiaCursorException ex) {
            System.err.println("Unexpected exception in main. Type code:"+ex.type);
            System.exit(1);
        }
        
        try {
            c.start();
            } catch(AppiaDuplicatedSessionsException ex) {
                System.err.println("Sessions binding strangely resulted in one single sessions occurring more than once in a channel");
                System.exit(1);
        }

        boardComponent.setKeyListener(new MyKeyListener((PlayerSession) s));

        connectButton=new Button("Connect to server");
        connectButton.addActionListener(new ConnectButtonListener(connectButton, (PlayerSession) s, host, port, boardComponent));
        add(connectButton);

        quitButton=new Button("Quit");
        quitButton.addActionListener(new QuitButtonListener());
        add(quitButton);

        connectButton.requestFocus();
        setVisible(true);

        Appia.run();
    }

    public void setBoard(Board board) {
        boardComponent.setBoard(board);
    }

    /**
    * It starts... Checks if the parameters are somewhat correct and starts the
    * player.
    *
    * @param args The command line parameter passed. It requires args[0]=HOST and args[1]=PORT
    */
    public static void main(String args[]) {
        if (args.length!=2) {
            System.out.println("Usage:\n\tGamePlayer <game server HOST> <game server PORT>");
            System.exit(1);
        }

        try    {
            java.net.InetAddress.getByName(args[0]);
        } catch (java.net.UnknownHostException e) {
            System.err.println("Unable to resolve host name ["+args[0]+"]");
            System.exit(1);
        }

        int port=0;
        try    {
            port=new Integer(args[1]).intValue();
            if (port<=0)
                throw new NumberFormatException();
        } catch (NumberFormatException e) {
            System.err.println("Port must be a Number");
            System.exit(1);
        }

        new GamePlayer(args[0], port);
    }
}