package stftr.server;

import java.io.*;
import appia.*;
import appia.protocols.tcpcomplete.*;
import stftr.Board;
import stftr.protocols.Game.*;
import stftr.protocols.Distribute.*;

/**
* This class is responsible for launching the Game Server.
*/
public class LaunchGameServer {
    /**
    * Launches the Game Server.
    *
    * @param filename The filename of the file that contains the board.
    * @param port The port where the server should run.
    */
    LaunchGameServer(String filename, int port) {        
        Layer[] qosLayers={
            new TcpCompleteLayer(),
            new DistributeLayer(),
            new GameLayer(new GameServer(getBoard(filename)), port)
        };

        QoS qos=null;
        
        try {
            qos=new QoS("stftr game server QoS", qosLayers);
        } catch(AppiaInvalidQoSException ex) {
            System.err.println("Invalid QoS");
            System.err.println(ex.getMessage());
            System.exit(1);
        }

        Channel c=qos.createUnboundChannel("stftr channel");
        Session s=qosLayers[qosLayers.length-1].createSession();
        ChannelCursor cc=c.getCursor();
        
        try {
            cc.top();
            cc.setSession(s);
        } catch(AppiaCursorException ex) {
            System.err.println("Unexpected exception in main. Type code:"+ex.type);
            System.exit(1);
        }
        
        try {
            c.start();
            } catch(AppiaDuplicatedSessionsException ex) {
                System.err.println("Sessions binding strangely resulted in one single sessions occurring more than once in a channel");
                System.exit(1);
        }

        Appia.run();
    }

    /**
    * Reads the board from a file.
    *
    * @param filename The filename of the file containing the board.
    */
    Board getBoard(String filename) {
        Board board=new Board();

        try {
            FileInputStream fis=new FileInputStream(new File(filename));
            for (int y=0; y<Board.HEIGHT; y++) {
                for (int x=0; x<Board.WIDTH; x++) {
                    char c;

                    while ((c=(char) fis.read())!='.' && c!='#')
                        ;

                    if (c=='#')
                        board.putBoard(x, y, Board.WALL, 0, 0);
                }
            }
        } catch (Exception e) {
            System.err.println("Error while trying to read the board from file ["+filename+"].");
            e.printStackTrace();
            System.exit(1);
        }

        return board;
    }

    /**
    * It starts...
    * Tests to see if parameters are correct and then launches the server.
    *
    * @param args The parameters given at command line.
    */
    public static void main(String args[]) {
        if (args.length!=2) {
            System.out.println("Usage:\n\tLaunchGameServer <board file> <port>");
            System.exit(1);
        } else {
            try {
                int i=new Integer(args[1]).intValue();
                if (i<=0)
                    throw new NumberFormatException();
                new LaunchGameServer(args[0], i);
            } catch (NumberFormatException ex) {
                System.out.println("<port> must be a positive integer (above 1025 if not an Administrator).");
            }
        }
    }
}